/*
 * © NHN Corp. All rights reserved.
 * NHN Corp. PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package com.toast.android.gamebase.sample.ui;

import android.app.Activity;
import android.content.Context;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Switch;
import android.widget.Toast;

import androidx.fragment.app.Fragment;

import com.toast.android.gamebase.Gamebase;
import com.toast.android.gamebase.base.push.PushConfiguration;
import com.toast.android.gamebase.sample.GamebaseManager;
import com.toast.android.gamebase.sample.R;
import com.toast.android.gamebase.sample.util.AppUtil;
import com.toast.android.gamebase.sample.util.Log;

public class PushFragment extends Fragment {

    private static final String TAG = PushFragment.class.getSimpleName();
    private static View mCurrentView = null;

    private static Switch mSwitchAdvertisement = null;
    private static Switch mSwitchAdvertisementNight = null;
    private static Switch mSwitchNotification = null;
    private static Switch mSwitchForeground = null;

    private static boolean mAgreeAdvertisement = false;
    private static boolean mAgreeAdvertisementNight = false;
    private static boolean mEnableNotification = false;
    private static boolean mEnableForeground = false;

    public PushFragment() {
        // Required empty public constructor
    }


    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {

        setHasOptionsMenu(true);

        mCurrentView = inflater.inflate(R.layout.fragment_push, container, false);

        getPushStatus();
        setPushAgreement();

        return mCurrentView;
    }


    ////////////////////////////////////////////////////////////////////////////////////////////////
    // Action Bar Item Setting
    @Override
    public void onCreateOptionsMenu(Menu menu, MenuInflater inflater) {
        menu.clear();
        super.onCreateOptionsMenu(menu, inflater);

        inflater.inflate(R.menu.main, menu);

        MenuItem logoutItem = menu.findItem(R.id.action_logout);
        logoutItem.setVisible(false);
        MenuItem closeItem = menu.findItem(R.id.action_activityClose);
        closeItem.setVisible(true);
        MenuItem logOpenItem = menu.findItem(R.id.action_log);
        logOpenItem.setVisible(true);

    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        ((MainActivity) getActivity()).actionItemEvent(item);
        return super.onOptionsItemSelected(item);
    }
    ////////////////////////////////////////////////////////////////////////////////////////////////

    private void setPushAgreement() {
        mSwitchAdvertisement = mCurrentView.findViewById(R.id.switch_adv);
        mSwitchAdvertisementNight = mCurrentView.findViewById(R.id.switch_adv_Night);
        mSwitchNotification = mCurrentView.findViewById(R.id.switch_notification);
        mSwitchForeground = mCurrentView.findViewById(R.id.switch_foreground_notification);

        mSwitchAdvertisement.setOnClickListener(v -> {
            mAgreeAdvertisement = mSwitchAdvertisement.isChecked();
            registerPush(mEnableNotification, mAgreeAdvertisement, mAgreeAdvertisementNight, mSwitchAdvertisement);
        });

        mSwitchAdvertisementNight.setOnClickListener(v -> {
            mAgreeAdvertisementNight = mSwitchAdvertisementNight.isChecked();
            registerPush(mEnableNotification, mAgreeAdvertisement, mAgreeAdvertisementNight, mSwitchAdvertisementNight);
        });

        mSwitchNotification.setOnClickListener(v -> {
            mEnableNotification = mSwitchNotification.isChecked();
            registerPush(mEnableNotification, mAgreeAdvertisement, mAgreeAdvertisementNight, mSwitchNotification);
        });

        mSwitchForeground.setOnClickListener(v -> {
            mEnableForeground = mSwitchForeground.isChecked();
            GamebaseManager.enablePushForeground(getActivity(), mEnableForeground, null);
        });

        mSwitchNotification.setOnCheckedChangeListener((buttonView, isChecked) -> {
            if (isChecked) {
                mSwitchAdvertisement.setEnabled(true);
                mSwitchForeground.setEnabled(true);
            } else {
                mSwitchAdvertisement.setEnabled(false);
                mSwitchAdvertisementNight.setEnabled(false);
                mSwitchForeground.setEnabled(false);
            }
        });

        mSwitchAdvertisement.setOnCheckedChangeListener((buttonView, isChecked) -> {
            if (isChecked) {
                mSwitchAdvertisementNight.setEnabled(true);
            } else {
                mSwitchAdvertisementNight.setEnabled(false);
            }
        });
    }

    private void getPushStatus() {
        final Activity activity = getActivity();
        final Context context = getContext();
        if (activity == null || context == null) {
            return;
        }
        AppUtil.showProgressDialog(activity, "Get Push Agreement Status ...");

        GamebaseManager.queryTokenInfo(activity, (data, exception) -> {
            boolean hasPushConfiguration = false;
            PushConfiguration pushConfiguration = GameCodeManager.loadPushConfiguration(context);
            if (Gamebase.isSuccess(exception)) {
                hasPushConfiguration = true;
                mAgreeAdvertisement = data.agreement.adAgreement;
                mAgreeAdvertisementNight = data.agreement.adAgreementNight;
                mEnableNotification = data.agreement.pushEnabled;
            } else if (pushConfiguration != null) {
                hasPushConfiguration = true;
                mAgreeAdvertisement = pushConfiguration.adAgreement;
                mAgreeAdvertisementNight = pushConfiguration.adAgreementNight;
                mEnableNotification = pushConfiguration.pushEnabled;
            }
            if (hasPushConfiguration) {
                mEnableForeground = Gamebase.Push.getNotificationOptions(context).isForegroundEnabled();
                activity.runOnUiThread(() -> applyPushSettingsToUI());

                AppUtil.dismissProgressDialog(activity);

                Log.d(TAG, "Agree Advertisement : " + mAgreeAdvertisement);
                Log.d(TAG, "Agree Advertisement Night : " + mAgreeAdvertisementNight);
                Log.d(TAG, "Enable Notification : " + mEnableNotification);
                Log.d(TAG, "Foreground Notification : " + mEnableForeground);
                Log.d(TAG, "Push Token : " + data.token);

                Gamebase.Util.showToast(activity, "Notification Enable : " + mEnableNotification +
                                "\nAdvertisement Agreement : " + mAgreeAdvertisement +
                                "\nNight Advertisement Agreement : " + mAgreeAdvertisementNight,
                        Toast.LENGTH_SHORT);
                return;
            }

            Gamebase.Util.showToast(activity, "Failed Get AdvertisementAgreement ", Toast.LENGTH_SHORT);
            Gamebase.Util.showAlert(activity, "Error", exception.toJsonString());
            AppUtil.dismissProgressDialog(activity);

            Log.d(TAG, "Failed Get AdvertisementAgreement ");
            Log.d(TAG, "exception Code : " + exception.getCode());
            Log.d(TAG, "exception Message : " + exception.getMessage());
            Log.d(TAG, "exception Domain : " + exception.getDomain());
            Log.d(TAG, "exception Detail Code : " + exception.getDetailCode());
            Log.d(TAG, "exception Detail Message: " + exception.getDetailMessage());
            Log.d(TAG, "exception Detail Domain: " + exception.getDetailDomain());
        });
    }

    private void applyPushSettingsToUI() {
        mSwitchAdvertisement.setChecked(mAgreeAdvertisement);
        mSwitchAdvertisementNight.setChecked(mAgreeAdvertisementNight);
        mSwitchNotification.setChecked(mEnableNotification);
        mSwitchForeground.setChecked(mEnableForeground);

        if (!mEnableNotification) {
            mSwitchAdvertisement.setEnabled(false);
            mSwitchAdvertisementNight.setEnabled(false);
            mSwitchForeground.setEnabled(false);
        } else if (!mAgreeAdvertisement) {
            mSwitchAdvertisementNight.setEnabled(false);
        }
    }

    private void registerPush(final boolean enable,
                              final boolean agree,
                              final boolean agreeNight,
                              final Switch currentSwitch) {
        final Activity activity = getActivity();
        final Context context = getContext();
        if (activity == null || context == null) {
            return;
        }
        AppUtil.showProgressDialog(activity, "Register push ...");

        final PushConfiguration configuration = PushConfiguration.newBuilder(enable, agree, agreeNight).build();
        GameCodeManager.savePushConfiguration(context, configuration);
        GamebaseManager.registerPush(activity, configuration, exception -> {
            if (Gamebase.isSuccess(exception)) {
                Gamebase.Util.showToast(activity, "Success Register Push", Toast.LENGTH_SHORT);
                AppUtil.dismissProgressDialog(activity);

                getPushStatus();
                return;
            }

            activity.runOnUiThread(() -> currentSwitch.setChecked(false));
            Gamebase.Util.showAlert(activity, "Error", exception.toJsonString());
            String errorMessage = "Failed Register Push - Notification Enable : " + enable + " | Advertisement Agreement : " + agree + " | Night Advertisement Agreement : " + agreeNight;
            Gamebase.Util.showToast(activity, errorMessage, Toast.LENGTH_SHORT);
            AppUtil.dismissProgressDialog(activity);

            Log.d(TAG, errorMessage);
            Log.d(TAG, "exception Code : " + exception.getCode());
            Log.d(TAG, "exception Message : " + exception.getMessage());
            Log.d(TAG, "exception Domain : " + exception.getDomain());
            Log.d(TAG, "exception Detail Code : " + exception.getDetailCode());
            Log.d(TAG, "exception Detail Message: " + exception.getDetailMessage());
            Log.d(TAG, "exception Detail Domain: " + exception.getDetailDomain());
        });
    }

}
